@@ -2,19 +2,13 @@ use crate::types::*;
 use fuel_core_interfaces::model::ArcTx;
 use std::{cmp, collections::BTreeMap};
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Default, Clone)]
 pub struct PriceSort {
     /// all transactions sorted by min/max value
     pub sort: BTreeMap<PriceSortKey, ArcTx>,
 }
 
 impl PriceSort {
-    pub fn new() -> Self {
-        Self {
-            sort: BTreeMap::new(),
-        }
-    }
-
     pub fn remove(&mut self, tx: &ArcTx) {
         self.sort.remove(&PriceSortKey::new(tx));
     }
@@ -37,12 +31,6 @@ impl PriceSort {
     }
 }
 
-impl Default for PriceSort {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
 #[derive(Clone, Debug)]
 pub struct PriceSortKey {
     price: GasPrice,
