@@ -1,27 +1,30 @@
-use crate::containers::dependency::Dependency;
-use crate::Error;
-use crate::{containers::price_sort::PriceSort, types::*, Config};
-use fuel_core_interfaces::model::{ArcTx, TxInfo};
-use fuel_core_interfaces::txpool::TxPoolDb;
+use crate::{
+    containers::{dependency::Dependency, price_sort::PriceSort},
+    types::*,
+    Config, Error,
+};
+use fuel_core_interfaces::{
+    model::{ArcTx, TxInfo},
+    txpool::TxPoolDb,
+};
 use std::collections::HashMap;
-use std::sync::Arc;
 
 #[derive(Debug, Clone)]
 pub struct TxPool {
     by_hash: HashMap<TxId, TxInfo>,
     by_gas_price: PriceSort,
     by_dependency: Dependency,
-    config: Arc<Config>,
+    config: Config,
 }
 
 impl TxPool {
-    pub fn new(config: Arc<Config>) -> Self {
+    pub fn new(config: Config) -> Self {
         let max_depth = config.max_depth;
         Self {
             by_hash: HashMap::new(),
             by_gas_price: PriceSort::default(),
-            config,
             by_dependency: Dependency::new(max_depth),
+            config,
         }
     }
     pub fn txs(&self) -> &HashMap<TxId, TxInfo> {
@@ -38,6 +41,11 @@ impl TxPool {
             return Err(Error::NoMetadata.into());
         }
 
+        // verify gas price is at least the minimum
+        self.verify_tx_min_gas_price(&tx)?;
+        // verify byte price is at least the minimum
+        self.verify_tx_min_byte_price(&tx)?;
+
         if self.by_hash.contains_key(&tx.id()) {
             return Err(Error::NotInsertedTxKnown.into());
         }
@@ -110,6 +118,20 @@ impl TxPool {
         }
         Vec::new()
     }
+
+    fn verify_tx_min_gas_price(&mut self, tx: &Transaction) -> Result<(), Error> {
+        if tx.gas_price() < self.config.min_gas_price {
+            return Err(Error::NotInsertedGasPriceTooLow);
+        }
+        Ok(())
+    }
+
+    fn verify_tx_min_byte_price(&mut self, tx: &Transaction) -> Result<(), Error> {
+        if tx.byte_price() < self.config.min_byte_price {
+            return Err(Error::NotInsertedBytePriceTooLow);
+        }
+        Ok(())
+    }
 }
 
 #[cfg(test)]
@@ -119,14 +141,15 @@ pub mod tests {
     use fuel_core_interfaces::{db::helpers::*, model::CoinStatus};
     use fuel_tx::UtxoId;
     use std::cmp::Reverse;
+    use std::sync::Arc;
 
     #[tokio::test]
     async fn simple_insertion() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
 
         let mut txpool = TxPool::new(config);
         let out = txpool.insert(tx1, &db).await;
@@ -135,13 +158,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn simple_dependency_tx1_tx2() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID2;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -153,13 +176,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn faulty_t2_collided_on_contract_id_from_tx1() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID_FAULTY2;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -174,14 +197,14 @@ pub mod tests {
 
     #[tokio::test]
     async fn fails_to_insert_tx2_with_missing_utxo_dependency_on_faulty_tx1() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_faulty_hash = *TX_ID_FAULTY1;
         let tx2_hash = *TX_ID2;
 
-        let tx1_faulty = Arc::new(DummyDB::dummy_tx(tx1_faulty_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx1_faulty = Arc::new(DummyDb::dummy_tx(tx1_faulty_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -194,11 +217,11 @@ pub mod tests {
 
     #[tokio::test]
     async fn not_inserted_known_tx() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1 = *TX_ID1;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1));
 
         let mut txpool = TxPool::new(config);
 
@@ -214,11 +237,11 @@ pub mod tests {
 
     #[tokio::test]
     async fn try_to_insert_tx2_missing_utxo() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx2_hash = *TX_ID2;
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -229,17 +252,19 @@ pub mod tests {
 
     #[tokio::test]
     async fn tx1_try_to_use_spend_coin() {
-        let config = Arc::new(Config::default());
-        let mut db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         // mark utxo as spend
-        db.coins
+        db.data
+            .lock()
+            .coins
             .get_mut(&UtxoId::new(*TX_ID_DB1, 0))
             .unwrap()
             .status = CoinStatus::Spent;
 
         let tx1_hash = *TX_ID1;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -250,13 +275,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn more_priced_tx3_removes_tx1() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx3_hash = *TX_ID3;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx3 = Arc::new(DummyDB::dummy_tx(tx3_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx3 = Arc::new(DummyDb::dummy_tx(tx3_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -272,13 +297,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn underpriced_tx1_not_included_coin_colision() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx3_hash = *TX_ID3;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx3 = Arc::new(DummyDB::dummy_tx(tx3_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx3 = Arc::new(DummyDb::dummy_tx(tx3_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -292,13 +317,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn overpriced_tx5_contract_input_not_inserted() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx5_hash = *TX_ID5;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let mut tx5 = DummyDB::dummy_tx(tx5_hash);
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let mut tx5 = DummyDb::dummy_tx(tx5_hash);
         tx5.set_gas_price(tx1.gas_price() + 1);
         tx5.precompute_metadata();
         let tx5 = Arc::new(tx5);
@@ -315,13 +340,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn dependent_tx5_contract_input_inserted() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx5_hash = *TX_ID5;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx5 = Arc::new(DummyDB::dummy_tx(tx5_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx5 = Arc::new(DummyDb::dummy_tx(tx5_hash));
         let mut txpool = TxPool::new(config);
 
         let out = txpool.insert(tx1, &db).await;
@@ -332,15 +357,15 @@ pub mod tests {
 
     #[tokio::test]
     async fn more_priced_tx3_removes_tx1_and_dependent_tx2() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID2;
         let tx3_hash = *TX_ID3;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
-        let tx3 = Arc::new(DummyDB::dummy_tx(tx3_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
+        let tx3 = Arc::new(DummyDb::dummy_tx(tx3_hash));
 
         let mut txpool = TxPool::new(config);
 
@@ -358,16 +383,17 @@ pub mod tests {
 
     #[tokio::test]
     async fn tx_limit_hit() {
-        let config = Arc::new(Config {
+        let config = Config {
             max_tx: 1,
             max_depth: 10,
-        });
-        let db = DummyDB::filled();
+            ..Default::default()
+        };
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID2;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
         let mut txpool = TxPool::new(config);
 
         let out = txpool.insert(tx1, &db).await;
@@ -379,16 +405,17 @@ pub mod tests {
 
     #[tokio::test]
     async fn tx2_depth_hit() {
-        let config = Arc::new(Config {
+        let config = Config {
             max_tx: 10,
             max_depth: 1,
-        });
-        let db = DummyDB::filled();
+            ..Default::default()
+        };
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID2;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
         let mut txpool = TxPool::new(config);
 
         let out = txpool.insert(tx1, &db).await;
@@ -400,15 +427,15 @@ pub mod tests {
 
     #[tokio::test]
     async fn sorted_out_tx1_2_4() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID2;
         let tx4_hash = *TX_ID4;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
-        let tx4 = Arc::new(DummyDB::dummy_tx(tx4_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
+        let tx4 = Arc::new(DummyDb::dummy_tx(tx4_hash));
         let mut txpool = TxPool::new(config);
 
         let out = txpool.insert(tx1, &db).await;
@@ -428,13 +455,13 @@ pub mod tests {
 
     #[tokio::test]
     async fn find_dependent_tx1_tx2() {
-        let config = Arc::new(Config::default());
-        let db = DummyDB::filled();
+        let config = Config::default();
+        let db = DummyDb::filled();
 
         let tx1_hash = *TX_ID1;
         let tx2_hash = *TX_ID2;
-        let tx1 = Arc::new(DummyDB::dummy_tx(tx1_hash));
-        let tx2 = Arc::new(DummyDB::dummy_tx(tx2_hash));
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+        let tx2 = Arc::new(DummyDb::dummy_tx(tx2_hash));
         let mut txpool = TxPool::new(config);
 
         let out = txpool.insert(tx1, &db).await;
@@ -452,4 +479,78 @@ pub mod tests {
         assert_eq!(list[0].id(), tx1_hash, "Tx1 should be first.");
         assert_eq!(list[1].id(), tx2_hash, "Tx2 should be second.");
     }
+
+    #[tokio::test]
+    async fn tx_at_least_min_gas_price_is_insertable() {
+        let config = Config {
+            min_gas_price: TX1_GAS_PRICE,
+            ..Config::default()
+        };
+        let db = DummyDb::filled();
+
+        let tx1_hash = *TX_ID1;
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+
+        let mut txpool = TxPool::new(config);
+
+        let out = txpool.insert(tx1, &db).await;
+        assert!(out.is_ok(), "Tx1 should be OK, get err:{:?}", out);
+    }
+
+    #[tokio::test]
+    async fn tx_below_min_gas_price_is_not_insertable() {
+        let config = Config {
+            min_gas_price: TX1_GAS_PRICE + 1,
+            ..Config::default()
+        };
+        let db = DummyDb::filled();
+
+        let tx1_hash = *TX_ID1;
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+
+        let mut txpool = TxPool::new(config);
+
+        let err = txpool.insert(tx1, &db).await.err().unwrap();
+        assert!(matches!(
+            err.root_cause().downcast_ref::<Error>().unwrap(),
+            Error::NotInsertedGasPriceTooLow
+        ));
+    }
+
+    #[tokio::test]
+    async fn tx_above_min_byte_price_is_insertable() {
+        let config = Config {
+            min_byte_price: TX1_BYTE_PRICE,
+            ..Config::default()
+        };
+        let db = DummyDb::filled();
+
+        let tx1_hash = *TX_ID1;
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+
+        let mut txpool = TxPool::new(config);
+
+        let out = txpool.insert(tx1, &db).await;
+        assert!(out.is_ok(), "Tx1 should be OK, get err:{:?}", out);
+    }
+
+    #[tokio::test]
+    async fn tx_below_min_byte_price_is_not_insertable() {
+        let config = Config {
+            min_byte_price: TX1_BYTE_PRICE + 1,
+            ..Config::default()
+        };
+        let db = DummyDb::filled();
+
+        let tx1_hash = *TX_ID1;
+        let tx1 = Arc::new(DummyDb::dummy_tx(tx1_hash));
+
+        let mut txpool = TxPool::new(config);
+
+        let err = txpool.insert(tx1, &db).await.err().unwrap();
+        assert!(matches!(
+            err.root_cause().downcast_ref::<Error>().unwrap(),
+            Error::NotInsertedBytePriceTooLow
+        ));
+    }
 }
